#include <gnome.h>

struct params {
  gchar *name;
  glong value;
  GSList *button_list;
  GtkWidget *label,*on,*off;
};

gchar *parameter_names[] = {
#define CANVAS 0
  "Display canvas events",
#define ITEMS 1
  "Display item events",
#define GROUPS 2
  "Display group events",
#define ENTERS 3
  "Display enter events",
#define LEAVES 4
  "Display leave events",
#define PRESSES 5
  "Display press events",
#define RELEASES 6
  "Display release events",
#define EOL 7
};

struct params *parameters;

static gulong seq = 0;

void create_test_items(GnomeCanvas *canvas,gint width,gint height);
gint event_cb(GtkWidget *widget,GdkEvent *event,gpointer data);
gint press_cb(GtkWidget *widget,gpointer data);
gint release_cb(GtkWidget *widget,gpointer data);
gint toggle_cb(GtkToggleButton *togglebutton,gpointer user_data);

gint main(gint argc,gchar *argv[]) {
  GtkWidget *appwindow,*parmwindow;
  GtkWidget *canvas;
  GtkWidget *table;
  gint i;

  gnome_init("Event tests","0.0.0",argc,argv);

  parameters = g_new(struct params,EOL);
  parmwindow = gnome_app_new("testparms","Event test parameters");
  table = gtk_table_new(3,EOL,FALSE);
  for (i=0;i<EOL;i++) {
    parameters[i].name = parameter_names[i];
    parameters[i].value = 1;
    parameters[i].label = gtk_label_new(parameters[i].name);
    parameters[i].button_list = NULL;
    parameters[i].on = gtk_radio_button_new_with_label(
      parameters[i].button_list,"On");
    gtk_object_set(GTK_OBJECT(parameters[i].on),"active",FALSE);
    gtk_signal_connect(GTK_OBJECT(parameters[i].on),"toggled",
                       GTK_SIGNAL_FUNC(toggle_cb),
                       GINT_TO_POINTER(i));
    parameters[i].off = gtk_radio_button_new_with_label(
      parameters[i].button_list,"Off");
    gtk_object_set(GTK_OBJECT(parameters[i].off),"active",TRUE);
    gtk_signal_connect(GTK_OBJECT(parameters[i].on),"toggled",
                       GTK_SIGNAL_FUNC(toggle_cb),
                       GINT_TO_POINTER(i));
    gtk_table_attach(GTK_TABLE(table),parameters[i].label,
      0,1,i,i+1,0,0,0,0);
    gtk_table_attach(GTK_TABLE(table),parameters[i].on,
      1,2,i,i+1,0,0,0,0);
    gtk_table_attach(GTK_TABLE(table),parameters[i].off,
      2,3,i,i+1,0,0,0,0);
  }
  gnome_app_set_contents(GNOME_APP(parmwindow),table);
//  gtk_widget_show_all(parmwindow);


  appwindow = gnome_app_new("test","Event tests");
  canvas = gnome_canvas_new();
  gtk_signal_connect_after(GTK_OBJECT(canvas),"event",
                     GTK_SIGNAL_FUNC(event_cb),NULL);
  gtk_signal_connect_after(GTK_OBJECT(canvas),"button_press_event",
                     GTK_SIGNAL_FUNC(press_cb),NULL);
  gtk_signal_connect_after(GTK_OBJECT(canvas),"button_release_event",
                     GTK_SIGNAL_FUNC(release_cb),NULL);
  gtk_widget_set_usize(canvas,400,400);
  gnome_app_set_contents(GNOME_APP(appwindow),canvas);
  gtk_widget_show_all(appwindow);

  create_test_items(GNOME_CANVAS(canvas),400,400);

  gtk_main();
  return 0;
}

void create_test_items(GnomeCanvas *canvas,gint width,gint height) {
  GnomeCanvasGroup *groups[4];
  GnomeCanvasItem *items[4];
  gdouble x,y,w,h;
  gint i;

//  x = width/2;y = height/2;
  x = y = 0;
  w = width - 50;h = height - 50;

  /* N times */
  for (i=0;i<4;i++) {
    /* if root, use root group */
    if (!i)
      groups[i] = GNOME_CANVAS_GROUP(canvas->root);
    else
      groups[i] = GNOME_CANVAS_GROUP(gnome_canvas_item_new(
        groups[i-1],gnome_canvas_group_get_type(),NULL));
    /* attach a name and signal */
    gtk_object_set_data(GTK_OBJECT(groups[i]),"num",GINT_TO_POINTER(i));
    gtk_signal_connect(GTK_OBJECT(groups[i]),"event",
                       GTK_SIGNAL_FUNC(event_cb),groups[i]);

    /* now create rectangle */
    items[i] = gnome_canvas_item_new(groups[i],gnome_canvas_rect_get_type(),
      "width_units",1.0,"fill_color","white","outline_color","black",
      "x1",x-(w/2),"y1",y-(h/2),"x2",x+(w/2),"y2",y+(h/2),NULL);

    /* attach a name and signal */
    gtk_object_set_data(GTK_OBJECT(items[i]),"num",GINT_TO_POINTER(i));
    gtk_signal_connect(GTK_OBJECT(items[i]),"event",
                       GTK_SIGNAL_FUNC(event_cb),items[i]);

    /* shrink */
    w -= 50;h -= 50;
  }
}

gint event_cb(GtkWidget *widget,GdkEvent *event,gpointer data) {
  gint num = GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(widget),"num"));
  gchar *name;

  if (GNOME_IS_CANVAS(widget)) {
    if (!parameters[CANVAS].value) return FALSE;
    name = g_strdup_printf("canvas");
  } else if (GNOME_IS_CANVAS_GROUP(widget)) {
    if (!parameters[ITEMS].value) return FALSE;
    name = g_strdup_printf("group %d",num);
  } else {
    if (!parameters[GROUPS].value) return FALSE;
    name = g_strdup_printf("item %d",num);
  }

  switch (event->type) {
    case GDK_ENTER_NOTIFY:
      if (parameters[ENTERS].value)
        g_print("%4d entered %s\n",seq++,name);
      break;
    case GDK_LEAVE_NOTIFY:
      if (parameters[LEAVES].value)
        g_print("%4d left %s\n",seq++,name);
      break;
    case GDK_BUTTON_PRESS:
      if (parameters[PRESSES].value)
        g_print("%4d pressed button in %s\n",seq++,name);
      break;
    case GDK_BUTTON_RELEASE:
      if (parameters[RELEASES].value)
        g_print("%4d released button in %s\n",seq++,name);
      break;
    default:
      return FALSE;
      break;
  }

  if (GNOME_IS_CANVAS(widget))
    return FALSE;
  else
    return TRUE;
}

gint toggle_cb(GtkToggleButton *togglebutton,gpointer user_data) {
  gint num = GPOINTER_TO_INT(user_data);

  g_print("toggled %d\n",num);
  parameters[num].value = !parameters[num].value;
}

gint press_cb(GtkWidget *widget,gpointer data) {
  g_print("%4d button pressed in canvas\n",seq++);
  return TRUE;
}

gint release_cb(GtkWidget *widget,gpointer data) {
  g_print("%4d button pressed in canvas\n",seq++);
  return TRUE;
}

