/* Gnome-Streamer
 * Copyright (C) <1999> Erik Walthinsen <omega@cse.ogi.edu>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#ifndef __GST_BUFFER_H__
#define __GST_BUFFER_H__


#include <gst/gstobject.h>
#include <gst/gstbuffer.h>


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


#define GST_TYPE_BUFFER \
  (gst_buffer_get_type())
#define GST_BUFFER(obj) \
  (GTK_CHECK_CAST((obj),GST_TYPE_BUFFER,GstBuffer))
#define GST_BUFFER_CLASS(klass) \
  (GTK_CHECK_CLASS_CAST((klass),GST_TYPE_BUFFER,GstBufferClass))
#define GST_IS_BUFFER(obj) \
  (GTK_CHECK_TYPE((obj),GST_TYPE_BUFFER))
#define GST_IS_BUFFER_CLASS(obj) \
  (GTK_CHECK_CLASS_TYPE((klass),GST_TYPE_BUFFER)))

typedef struct _GstBuffer GstBuffer;
typedef struct _GstBufferClass GstBufferClass;

struct _GstBuffer {
  GstObject object;

  gint refcount;

  guint32 type;
  guint16 flags;

  guchar *data;
  guint32 datasize;
  guint32 srcoffset;

  void *meta;
};

struct _GstBufferClass {
  GstObjectClass parent_class;
};

GtkType gst_buffer_get_type(void);
GstBuffer *gst_buffer_new();
void gst_buffer_destroy(GstBuffer *buffer);

void gst_buffer_ref(GstBuffer *buffer);
void gst_buffer_ref_by_count(GstBuffer *buffer,gint count);
void gst_buffer_unref(GstBuffer *buffer);

#ifdef __cplusplus
}
#endif /* __cplusplus */


#endif /* __GST_BUFFER_H__ */     

