/* Gnome-Streamer   
 * Copyright (C) <1999> Erik Walthinsen <omega@cse.ogi.edu>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */



#ifndef __GST_META_H__
#define __GST_META_H__

#include <glib.h> 
 
 
#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


typedef enum {
  GST_META_FREEABLE             = 1 << 0,
} GstMetaFlags;

#define GST_META(meta) ((GstMeta *)(meta))

typedef struct _GstMeta GstMeta;

struct _GstMeta {
  guint16 type;
  guint16 size;
  guint16 flags;
  guint16 refcount;

  GstMeta *(*copy) (GstMeta *meta);
  void (*destroy) (GstMeta *meta);
};

#define GST_META_FLAGS(meta) (GST_META(meta)->flags)
#define GST_META_SET_FLAGS(meta,flag) G_STMT_START{ (GST_META_FLAGS(meta) |= (flag)); }G_STMT_END
#define GST_META_UNSET_FLAGS(meta,flag) G_STMT_START{ (GST_META_FLAGS(meta) &= ~(flag)); }G_STMT_END
#define GST_META_IS_SET(meta,flag) G_STMT_START{ (GST_META_FLAGS(meta) && (flag)); }G_STMT_END

#define GST_META_TYPE(meta) (GST_META(meta)->type)
#define GST_META_IS_TYPE(meta,type) ((GST_META(meta)->type) == (type))


/* refcounting functions */
void gst_meta_ref(GstMeta *meta);
void gst_meta_unref(GstMeta *meta);

/* copy-on-write handler, returns potentially new meta */
GstMeta *gst_meta_cow(GstMeta *meta);

#ifdef __cplusplus
}
#endif /* __cplusplus */
 
 
#endif /* __GST_BUFFER_H__ */

