/* Gnome-Streamer
 * Copyright (C) <1999> Erik Walthinsen <omega@cse.ogi.edu>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <volume.h>


GstElementDetails volume_details = {
  "Volume control",
  "Filter/Effect",
  "Digitally control the volume of a signal",
  VERSION,
  "Erik Walthinsen <omega@cse.ogi.edu>",
  "(C) 1999",
};


/* Volume signals and args */
enum {
  /* FILL ME */
  LAST_SIGNAL
};

enum {
  ARG_0,
  ARG_ACTIVE,
  ARG_VOLUME
};


static void gst_volume_class_init(GstVolumeClass *klass);
static void gst_volume_init(GstVolume *volume);
static void gst_volume_set_arg(GtkObject *object,GtkArg *arg,guint id);
static void gst_volume_get_arg(GtkObject *object,GtkArg *arg,guint id);


static GstFilterClass *parent_class = NULL;
static guint gst_volume_signals[LAST_SIGNAL] = { 0 };

GtkType
gst_volume_get_type(void) {
  static GtkType volume_type = 0;

  if (!volume_type) {
    static const GtkTypeInfo volume_info = {
      "GstVolume",
      sizeof(GstVolume),
      sizeof(GstVolumeClass),
      (GtkClassInitFunc)gst_volume_class_init,
      (GtkObjectInitFunc)gst_volume_init,
      (GtkArgSetFunc)gst_volume_set_arg,
      (GtkArgGetFunc)gst_volume_get_arg,
      (GtkClassInitFunc)NULL,
    };
    volume_type = gtk_type_unique(GST_TYPE_FILTER,&volume_info);
  }
  return volume_type;
}

static void gst_volume_class_init(GstVolumeClass *klass) {
  GtkObjectClass *gtkobject_class;
  GstFilterClass *gstfilter_class;

  gtkobject_class = (GtkObjectClass*)klass;
  gstfilter_class = (GstFilterClass*)klass;

  parent_class = gtk_type_class(GST_TYPE_FILTER);

  gtk_object_add_arg_type("GstVolume::active", GTK_TYPE_INT,
                          GTK_ARG_READWRITE, ARG_ACTIVE);
  gtk_object_add_arg_type("GstVolume::volume", GTK_TYPE_FLOAT,
                          GTK_ARG_READWRITE, ARG_VOLUME);

  gtkobject_class->set_arg = gst_volume_set_arg;  
  gtkobject_class->get_arg = gst_volume_get_arg;
}

static void gst_volume_init(GstVolume *volume) {
  volume->sinkpad = gst_pad_new("sink",GST_PAD_SINK);
  gst_element_add_pad(GST_ELEMENT(volume),volume->sinkpad);
  gst_pad_set_chain_function(volume->sinkpad,gst_volume_chain);
  volume->srcpad = gst_pad_new("src",GST_PAD_SRC);
  gst_element_add_pad(GST_ELEMENT(volume),volume->srcpad);

  volume->active = FALSE;
  volume->volume = 1.0;
}

GstElement *gst_volume_new(gchar *name) {
  GstElement *volume = GST_ELEMENT(gtk_type_new(GST_TYPE_VOLUME));
  gst_object_set_name(GST_OBJECT(volume),name);
  return volume;
}

void gst_volume_chain(GstPad *pad,GstBuffer *buf) {
  GstVolume *volume;
  gint16 *data;
  gint16 factor;
  int i;
  guint64 start,stop;

  g_return_if_fail(pad != NULL);
  g_return_if_fail(GST_IS_PAD(pad));
  g_return_if_fail(buf != NULL);
  g_return_if_fail(GST_IS_BUFFER(buf));

  volume = GST_VOLUME(pad->parent);
  g_return_if_fail(volume != NULL);
  g_return_if_fail(GST_IS_VOLUME(volume));

  if (buf->meta)
    memcpy(&volume->meta,buf->meta,sizeof(volume->meta));

  if (volume->active) {
    if (volume->meta.format == AFMT_S16_LE) {
//      read_tsc(&start);
      data = (gint16 *)buf->data;
//      factor = (gint16)(0xffffL * volume->volume);
      for (i=0;i<buf->datasize/2;i++)
        data[i] = (float)data[i] * volume->volume;
//        data[i] = (data[i] * factor) / 0xffff;
//      read_tsc(&stop);
//      g_print("took %Ld clocks to set volume\n",stop-start);
    }
  }

  gst_trace_add_entry(NULL,0,buf,"volume: set volume");

  gst_pad_push(volume->srcpad,buf);
}

static void gst_volume_set_arg(GtkObject *object,GtkArg *arg,guint id) {
  GstVolume *volume;

  /* it's not null if we got it, but it might not be ours */
  g_return_if_fail(GST_IS_VOLUME(object));
  volume = GST_VOLUME(object);

  switch(id) {
    case ARG_ACTIVE:
      volume->active = GTK_VALUE_INT(*arg);
      break;
    case ARG_VOLUME:
      volume->volume = GTK_VALUE_FLOAT(*arg);
      break;
    default:
      break;
  }
}

static void gst_volume_get_arg(GtkObject *object,GtkArg *arg,guint id) {
  GstVolume *volume;

  /* it's not null if we got it, but it might not be ours */
  g_return_if_fail(GST_IS_VOLUME(object));
  volume = GST_VOLUME(object);

  switch (id) {
    case ARG_ACTIVE:
      GTK_VALUE_INT(*arg) = volume->active;
      break;
    case ARG_VOLUME:
      GTK_VALUE_FLOAT(*arg) = volume->volume;
      break;
    default:
      arg->type = GTK_TYPE_INVALID;
      break;
  }
}

GstPlugin *plugin_init(GModule *module) {
  GstPlugin *plugin;
  GstElementFactory *factory;
  guint16 rawtype;

  if (gst_plugin_find("volume") != NULL) return NULL;

  plugin = gst_plugin_new("volume");
  g_return_if_fail(plugin != NULL);

  factory = gst_elementfactory_new("volume",GST_TYPE_VOLUME,
                                   &volume_details);
  g_return_if_fail(factory != NULL);
  gst_plugin_add_factory(plugin,factory);
  g_print("volume: added 'volume' element\n");

  rawtype = gst_type_find_by_mime("audio/raw");
  g_return_if_fail(rawtype != 0);
  gst_type_add_sink(rawtype,factory);
  gst_type_add_src(rawtype,factory);

  return plugin;
}
