/* Gnome-Streamer
 * Copyright (C) <1999> Erik Walthinsen <omega@cse.ogi.edu>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <example.h>
#include <gst/gstplugin.h>

/* Example signals and args */
enum {
  /* FILL ME */
  LAST_SIGNAL
};

enum {
  ARG_0,
  ARG_ACTIVE
};


static void gst_example_class_init(GstExampleClass *klass);
static void gst_example_init(GstExample *example);
static void gst_example_set_arg(GtkObject *object,GtkArg *arg,guint id);
static void gst_example_get_arg(GtkObject *object,GtkArg *arg,guint id);


static GstFilterClass *parent_class = NULL;
static guint gst_example_signals[LAST_SIGNAL] = { 0 };

GtkType
gst_example_get_type(void) {
  static GtkType example_type = 0;

  if (!example_type) {
    static const GtkTypeInfo example_info = {
      "GstExample",
      sizeof(GstExample),
      sizeof(GstExampleClass),
      (GtkClassInitFunc)gst_example_class_init,
      (GtkObjectInitFunc)gst_example_init,
      (GtkArgSetFunc)gst_example_set_arg,
      (GtkArgGetFunc)gst_example_get_arg,
      (GtkClassInitFunc)NULL,
    };
    example_type = gtk_type_unique(GST_TYPE_FILTER,&example_info);
  }
  return example_type;
}

static void gst_example_class_init(GstExampleClass *klass) {
  GtkObjectClass *gtkobject_class;
  GstFilterClass *gstfilter_class;

  gtkobject_class = (GtkObjectClass*)klass;
  gstfilter_class = (GstFilterClass*)klass;

  parent_class = gtk_type_class(GST_TYPE_FILTER);

  gtk_object_add_arg_type("GstExample::active", GTK_TYPE_INT,
                          GTK_ARG_READWRITE, ARG_ACTIVE);

  gtkobject_class->set_arg = gst_example_set_arg;  
  gtkobject_class->get_arg = gst_example_get_arg;
}

static void gst_example_init(GstExample *example) {
  example->sinkpad = gst_pad_new("sink",GST_PAD_SINK);
  gst_element_add_pad(GST_ELEMENT(example),example->sinkpad);
  gst_pad_set_chain_function(example->sinkpad,gst_example_chain);
  example->srcpad = gst_pad_new("src",GST_PAD_SRC);
  gst_element_add_pad(GST_ELEMENT(example),example->srcpad);

  example->active = FALSE;
}

GstElement *gst_example_new(gchar *name) {
  GstElement *example = GST_ELEMENT(gtk_type_new(GST_TYPE_EXAMPLE));
  gst_object_set_name(GST_OBJECT(example),name);
  return example;
}

void gst_example_chain(GstPad *pad,GstBuffer *buf) {
  GstExample *example;
  guchar *data;
  gint i;
  gdouble avg,ldiff,rdiff,tmp,mul;

  g_return_if_fail(pad != NULL);
  g_return_if_fail(GST_IS_PAD(pad));
  g_return_if_fail(buf != NULL);
  g_return_if_fail(GST_IS_BUFFER(buf));

  example = GST_EXAMPLE(pad->parent);
  g_return_if_fail(example != NULL);
  g_return_if_fail(GST_IS_EXAMPLE(example));

  if (buf->meta)
    memcpy(&example->meta,buf->meta,sizeof(example->meta));

  if (example->active) {
    /* DO STUFF */
  }

  gst_pad_push(example->srcpad,buf);
}

static void gst_example_set_arg(GtkObject *object,GtkArg *arg,guint id) {
  GstExample *example;

  /* it's not null if we got it, but it might not be ours */
  g_return_if_fail(GST_IS_EXAMPLE(object));
  example = GST_EXAMPLE(object);

  switch(id) {
    case ARG_ACTIVE:
      example->active = GTK_VALUE_INT(*arg);
      g_print("example: set active to %d\n",example->active);
      break;
    default:
      break;
  }
}

static void gst_example_get_arg(GtkObject *object,GtkArg *arg,guint id) {
  GstExample *example;

  /* it's not null if we got it, but it might not be ours */
  g_return_if_fail(GST_IS_EXAMPLE(object));
  example = GST_EXAMPLE(object);

  switch (id) {
    case ARG_ACTIVE:
      GTK_VALUE_INT(*arg) = example->active;
      break;
    default:
      arg->type = GST_TYPE_INVALID;
      break;
  }
}

GstPlugin *plugin_init(GModule *module) {
  GstPlugin *plugin;
  GstElementFactory *factory;
  GstType *rawtype;

  factory = gst_elementfactory_new("example",GST_TYPE_EXAMPLE,gst_example_new);
  g_return_if_fail(factory != NULL);

  plugin = gst_plugin_new("example");
  g_return_if_fail(plugin != NULL);
  plugin->elements = g_list_append(plugin->elements,factory);

  rawtype = gst_type_get_by_mime("audio/raw");
  g_return_if_fail(rawtype != NULL);
  gst_type_add_sink(rawtype,factory);
  gst_type_add_src(rawtype,factory);

  g_print("example: added 'example' element\n");

  return plugin;
}
