/* Gnome-Streamer
 * Copyright (C) <1999> Erik Walthinsen <omega@cse.ogi.edu>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#ifndef __GST_XING_H__
#define __GST_XING_H__


#include <config.h>
#include <gst/gst.h>


/* Xing header */
#include <mhead.h>


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


#define GST_TYPE_XING \
  (gst_xing_get_type())
#define GST_XING(obj) \
  (GTK_CHECK_CAST((obj),GST_TYPE_XING,GstXing))
#define GST_XING_CLASS(klass) \
  (GTK_CHECK_CLASS_CAST((klass),GST_TYPE_XING,GstXing))
#define GST_IS_XING(obj) \
  (GTK_CHECK_TYPE((obj),GST_TYPE_XING))
#define GST_IS_XING_CLASS(obj) \
  (GTK_CHECK_CLASS_TYPE((klass),GST_TYPE_XING)))

typedef struct _GstXing GstXing;
typedef struct _GstXingClass GstXingClass;

struct _GstXing {
  GstFilter filter;

  /* pads */
  GstPad *sinkpad,*srcpad;

  /* most recent MP3 header */
  MPEG_HEAD header;		/* xing struct holding mp3 stuff */
  guint bitrate;		/* bitrate of current stream */
  gulong prevheader;		/* previous header word, to save time */
  guint framebytes;		/* bytes of last frame in prevbuf */

  /* in case we have something left over */
  GstBuffer *prevbuf;		/* the previous buffer */
  guint lastframe;		/* byte offset of last frame */
  guint lastframebytes;		/* bytes left over in last frame */
  guchar *framebuf;		/* memory region for reconstruction */
  gint framebufsize;		/* allocated size of region */

  /* error counters */
  gulong skipped_bytes;		/* how many bytes did we have to skip */
  gulong skipped_buffers;	/* how many buffers were useless */
};

struct _GstXingClass {
  GstFilterClass parent_class;
};

GtkType gst_xing_get_type(void);
GstElement *gst_xing_new(gchar *name);
void gst_xing_chain(GstPad *pad,GstBuffer *buf);


#ifdef __cplusplus
}
#endif /* __cplusplus */


#endif /* __GST_XING_H__ */
